package com.bank;

import junit.framework.TestCase;

import java.util.Hashtable;

public class AccountServiceTest extends TestCase
{
    private AccountService _accountService;

    class AccountDaoMock implements AccountDao
    {
        Hashtable accounts = new Hashtable();
        // This is where the hand tossed mock may be getting a bit more complex than it should be.
        // We may benefit from using a framework like EasyMock or JMock for this.

        public Account getAccount(int id)
        {
            Account account = (Account) accounts.get(id);

            if (account == null)
            {
                account = new Account();
                account.setId(id);
                account.setBalance(id * 5000);
            }

            return account;
        }

        public void save(Account account)
        {
            accounts.put(account.getId(), account);
        }
    }

    protected void setUp() throws Exception
    {
        _accountService = new AccountService();

        _accountService.setAccountDao(new AccountDaoMock());
    }

    public void testGetAccount()
    {
        Account account = _accountService.getAccount(1);
        assertEquals(1, account.getId());
        assertEquals(5000, account.getBalance());
    }

    public void testTransferFund()
    {
        Account account1 = _accountService.getAccount(1);
        Account account2 = _accountService.getAccount(2);

        assertEquals(5000, account1.getBalance());
        assertEquals(10000, account2.getBalance());

        _accountService.transferFund(1,  2,  1000);

        account1 = _accountService.getAccount(1);
        account2 = _accountService.getAccount(2);

        assertEquals(4000, account1.getBalance());
        assertEquals(11000, account2.getBalance());
    }

    public void testTransferMoreThanBalance()
    {
        Account account1 = _accountService.getAccount(1);
        Account account2 = _accountService.getAccount(2);

        assertEquals(5000, account1.getBalance());
        assertEquals(10000, account2.getBalance());

        try
        {
            _accountService.transferFund(1,  2,  10000);
            fail("InsufficientBalanceException  expected...");
        }
        catch(InsufficientBalanceException ex)
        {
            // OK
        }

        account1 = _accountService.getAccount(1);
        account2 = _accountService.getAccount(2);

        assertEquals(5000, account1.getBalance());
        assertEquals(10000, account2.getBalance());
    }    
}
