package com.bank;

import org.springframework.jdbc.object.MappingSqlQuery;
import org.springframework.jdbc.object.SqlUpdate;
import org.springframework.jdbc.core.SqlParameter;

import javax.sql.DataSource;
import java.sql.Types;
import java.sql.ResultSet;
import java.sql.SQLException;

public class JdbcAccountDao implements AccountDao
{
    private DataSource _dataSource;

    public void setDataSource(DataSource dataSource)
    {
        _dataSource = dataSource;
    }

    public Account getAccount(int id)
    {
        class AccountQuery extends MappingSqlQuery
        {
            public AccountQuery()
            {
                super(_dataSource, "SELECT * from accounts WHERE id = ?");
                super.declareParameter(new SqlParameter("id", Types.INTEGER));
                compile();
            }

            protected Object mapRow(ResultSet resultSet, int i) throws SQLException
            {
                Account account = new Account();
                account.setId(resultSet.getInt("id"));
                account.setBalance(resultSet.getInt("balance"));
                return account;
            }
        }

        AccountQuery query = new AccountQuery();
        return (Account) query.execute(new Object[] { id }).get(0);
    }

    public void save(Account account)
    {
        class AccountSaver extends SqlUpdate
        {
            public AccountSaver()
            {
                super.setDataSource(_dataSource);
                super.setSql("UPDATE accounts set balance = ? WHERE id = ?");
                super.declareParameter(new SqlParameter("balance", Types.INTEGER));
                super.declareParameter(new SqlParameter("id", Types.INTEGER));
                compile();
            }

            public int doUpdate(int id, int balance)
            {
                return update(new Object[] {balance, id});
            }
        }

        AccountSaver saver = new AccountSaver();
        saver.doUpdate(account.getId(), account.getBalance());
    }
}
